//--------------------------------------------------------------
/*

  ESP32 Co-Prozessor connect
  -> Only for ESP8266 and ESP32 Central Station!

  - send data over I2C to CO-MCU
  - Support LocoNet and XpressNet

  Copyright (c) by Philipp Gahtow, year 2021
  
*/
#if defined(ESP32_CoMCU)
#include <Wire.h>

// N.B. I2C 7-bit addresses can only be in the range of 0x08 -> 0x77
const int I2C_CoMCU_Address = 0x08;

//Report data is ready to read from slave
boolean DataRequest = false;

//****************************************************************  
//Call this in Setup routine
void CoMCU_init() {
  Wire.begin();        // join i2c bus (address optional for master)
  
  //receive PIN data ready to get
  pinMode(CoMCUDataReadyToReceivePin, INPUT);
}

//****************************************************************  
//Call this to send data to CoMCU - report ID for the interface
void CoMCU_send_data(byte *data, byte len, byte ID) {
  #if defined(ESP32I2CDEBUG)
    Debug.print("I2CTX L");
    Debug.print(len);
    Debug.print(" ID: 0x0");
    Debug.print(ID, HEX);
    Debug.print("-");
  #endif
  Wire.beginTransmission(I2C_CoMCU_Address); // transmit to slave device
  Wire.write((ID << 5) | (len & 0x1F));
  for (byte i = 0; i < len; i++) {
    Wire.write(data[i]);        // sends bytes
    #if defined(ESP32I2CDEBUG)
      Debug.print(" 0x");
      Debug.print(data[i], HEX);         // print the character
    #endif
  }
  Wire.endTransmission();    // stop transmitting
  #if defined(ESP32I2CDEBUG)
  Debug.println();
  #endif
}

//****************************************************************  
//Call this in each Loop round
void CoMCU_run() {
  //check if the slave has data?
  if (DataRequest == false && digitalRead(CoMCUDataReadyToReceivePin) == LOW) {
    Wire.requestFrom(I2C_CoMCU_Address, 1);    // request if slave has data?
    DataRequest = true;
  }

  //try to read data from  I2C?
  if (Wire.available()) {
    DataRequest = false;
    byte c = Wire.read(); // receive length of data
    if (c > 0) {    //Slave has data for us!
      //let the slave talk:
      Wire.requestFrom(I2C_CoMCU_Address, c & 0x1F); 
      #if defined(ESP32I2CDEBUG)
      Debug.print("I2CRX L");
      Debug.print(c & 0x1F);
      Debug.print(" ID: 0x0");
      Debug.print(c >> 5, HEX);
      Debug.print("-");
      #endif
      
      while (Wire.available()) { // slave may send less than requested
        c = Wire.read(); // receive a byte as character
        #if defined(ESP32I2CDEBUG)
        Debug.print(" 0x");
        Debug.print(c, HEX);         // print the character
        #endif
      }
      #if defined(ESP32I2CDEBUG)
      Debug.println();
      #endif
    }
    #if defined(ESP32I2CDEBUG)
    else Debug.println("I2CRX No Data!");
    #endif
  }
}

#endif
